@@ -3,9 +3,11 @@ use std::collections::HashMap;
 use std::convert::TryInto;
 use std::path::{Path, PathBuf};
 
-use tokio::fs::File;
+use async_compression::tokio::{bufread::GzipDecoder, write::GzipEncoder};
+use tokio::fs::{read_dir, File};
 use tokio::io::{AsyncRead, AsyncWriteExt};
 use tokio_stream::{Stream, StreamExt};
+use tokio_tar::Archive;
 use tokio_util::codec::{BytesCodec, FramedRead};
 use tracing::{debug, info, instrument, trace};
 
@@ -22,6 +24,9 @@ pub struct StandaloneRead {
     pub invoice_file: PathBuf,
     pub parcel_dir: PathBuf,
     pub parcels: Vec<PathBuf>,
+    // We keep the tarball tempdir in scope so it cleans up on drop
+    #[allow(dead_code)]
+    tarball_dir: Option<tempfile::TempDir>,
 }
 
 impl StandaloneRead {
@@ -46,26 +51,61 @@ impl StandaloneRead {
         I: TryInto<Id>,
         I::Error: Into<ClientError>,
     {
-        let base = base_path
-            .as_ref()
-            .join(bindle_id.try_into().map_err(|e| e.into())?.sha());
-        let invoice_file = base.join(INVOICE_FILE);
-        trace!(path = %invoice_file.display(), "Computed invoice file path");
-        let parcel_dir = base.join(PARCEL_DIR);
-        trace!(path = %parcel_dir.display(), "Listing parcels in parcels directory");
-        let stream = tokio::fs::read_dir(&parcel_dir).await?;
-        let parcels = tokio_stream::wrappers::ReadDirStream::new(stream)
-            .map(|res| res.map(|entry| entry.path()).map_err(|e| e.into()))
-            .collect::<Result<_>>()
-            .await?;
-        Ok(StandaloneRead {
-            invoice_file,
-            parcel_dir,
-            parcels,
-        })
+        parse_dir(
+            base_path
+                .as_ref()
+                .join(bindle_id.try_into().map_err(|e| e.into())?.sha()),
+            None,
+        )
+        .await
     }
 
-    // TODO: from a tarball
+    /// Returns a new StandaloneRead constructed using the given tarball.
+    ///
+    /// This will extract the tarball to a temporary location on disk. It will attempt to list all
+    /// parcel files, but will not provide any validation such as whether it is a regular file.
+    ///
+    /// ```no_run
+    /// use bindle::standalone::StandaloneRead;
+    /// # #[tokio::main]
+    /// # async fn main() {
+    /// StandaloneRead::new_from_tarball("/foo/bar.tar.gz").await.unwrap();
+    /// # }
+    /// ```
+    #[instrument(level = "trace", skip(tarball), fields(%path = tarball.as_ref().display()))]
+    pub async fn new_from_tarball<P: AsRef<Path>>(tarball: P) -> Result<StandaloneRead> {
+        let file = File::open(tarball).await?;
+
+        let tempdir = tokio::task::spawn_blocking(tempfile::TempDir::new)
+            .await
+            .map_err(|e| {
+                ClientError::Other(format!("Thread error when creating tempdir: {}", e))
+            })??;
+
+        let mut archive = Archive::new(GzipDecoder::new(tokio::io::BufReader::new(file)));
+
+        trace!("Unpacking tarball");
+        archive.unpack(tempdir.path()).await?;
+        trace!("Tarball unpacked, parsing directory");
+
+        // Get the path to the newly unpacked directory
+        let mut readdir = read_dir(tempdir.path()).await?;
+        // There should only be one entry, the expanded directory
+        let entry = readdir.next_entry().await?.ok_or_else(|| {
+            ClientError::Io(std::io::Error::new(
+                std::io::ErrorKind::NotFound,
+                "Unable to find unpacked directory",
+            ))
+        })?;
+        if !entry.metadata().await.map(|m| m.is_dir()).unwrap_or(false) {
+            return Err(ClientError::Other(format!(
+                "Found entry in temp directory that is not the expected directory: {}",
+                entry.file_name().to_string_lossy()
+            )));
+        }
+
+        parse_dir(entry.path().to_owned(), Some(tempdir)).await
+    }
 
     /// Push this standalone bindle to a bindle server using the given client. This function will
     /// automatically handle cases where the invoice or some of the parcels already exist on the
@@ -193,6 +233,29 @@ async fn create_or_get_invoice<T: TokenManager>(
     }
 }
 
+/// Helper function for parsing the directory and returning a StandaloneRead
+async fn parse_dir<P: AsRef<Path>>(
+    base_path: P,
+    tarball_dir: Option<tempfile::TempDir>,
+) -> Result<StandaloneRead> {
+    let base = tokio::fs::canonicalize(base_path).await?;
+    let invoice_file = base.join(INVOICE_FILE);
+    trace!(invoice_path = %invoice_file.display(), "Computed invoice file path");
+    let parcel_dir = base.join(PARCEL_DIR);
+    trace!(parcels_dir = %parcel_dir.display(), "Listing parcels in parcels directory");
+    let stream = read_dir(&parcel_dir).await?;
+    let parcels = tokio_stream::wrappers::ReadDirStream::new(stream)
+        .map(|res| res.map(|entry| entry.path()).map_err(|e| e.into()))
+        .collect::<Result<_>>()
+        .await?;
+    Ok(StandaloneRead {
+        invoice_file,
+        parcel_dir,
+        parcels,
+        tarball_dir,
+    })
+}
+
 /// A type that can write all bindle data to the configured location as a standalone bindle
 pub struct StandaloneWrite {
     base_path: PathBuf,
@@ -202,25 +265,33 @@ impl StandaloneWrite {
     /// Returns a new `StandaloneWrite` that can write all components of a bindle as a standalone
     /// bindle using the given base path and bindle ID
     ///
-    /// ```
+    /// ```no_run
     /// use bindle::standalone::StandaloneWrite;
     ///
-    /// StandaloneWrite::new("/foo/bar", "example.com/baz/1.0.0").unwrap();
+    /// # #[tokio::main]
+    /// # async fn main() {
+    /// StandaloneWrite::new("/foo/bar", "example.com/baz/1.0.0").await.unwrap();
+    /// # }
     /// ```
     ///
     /// In the above example, the `StandaloneWrite` will be configured to write all the bindle data
     /// into the `/foo/bar/187e908f466500c76c13953c3191fafa869c277e2689f451e92d75cda32452df`
     /// directory
-    pub fn new<P, I>(base_path: P, bindle_id: I) -> Result<StandaloneWrite>
+    pub async fn new<P, I>(base_path: P, bindle_id: I) -> Result<StandaloneWrite>
     where
         P: AsRef<Path>,
         I: TryInto<Id>,
         I::Error: Into<ClientError>,
     {
+        let path = base_path
+            .as_ref()
+            .join(bindle_id.try_into().map_err(|e| e.into())?.sha());
+
+        trace!(path = %path.display(), "Ensuring that directory exists");
+        tokio::fs::create_dir_all(&path).await?;
+
         Ok(StandaloneWrite {
-            base_path: base_path
-                .as_ref()
-                .join(bindle_id.try_into().map_err(|e| e.into())?.sha()),
+            base_path: tokio::fs::canonicalize(path).await?,
         })
     }
 
@@ -229,7 +300,40 @@ impl StandaloneWrite {
         self.base_path.as_ref()
     }
 
-    // TODO: From a tarball
+    /// Creates a tarball, consuming the `StandaloneWrite` and outputting the tarball at the given
+    /// path
+    #[instrument(level = "debug", skip(self, output_dir), fields(output_dir = %output_dir.as_ref().display()))]
+    pub async fn tarball(self, output_dir: impl AsRef<Path>) -> Result<()> {
+        // First, check if there is anything in the directory, if it is empty, then we abort
+        if read_dir(&self.base_path)
+            .await?
+            .next_entry()
+            .await?
+            .is_none()
+        {
+            return Err(ClientError::Other(
+                "Standalone bindle directory is empty. Unable to create tarball".to_string(),
+            ));
+        }
+        // We know the directory name will be the bindle ID, so grab it from there
+        // NOTE: The unwrap here is ok as we have canonicalized the path and created
+        let mut filename = self.base_path.file_name().unwrap().to_owned();
+        filename.push(".tar.gz");
+        let file = File::create(output_dir.as_ref().join(filename)).await?;
+        let encoder = GzipEncoder::new(file);
+
+        let mut builder = tokio_tar::Builder::new(encoder);
+        builder
+            .append_dir_all(self.base_path.file_name().unwrap(), &self.base_path)
+            .await?;
+
+        // Make sure everything is flushed to disk, otherwise we might miss closing data block
+        let mut encoder = builder.into_inner().await?;
+        encoder.flush().await?;
+        encoder.shutdown().await?;
+
+        Ok(())
+    }
 
     /// Writes the given invoice and `HashMap` of parcels (as readers). The key of the `HashMap`
     /// should be the SHA of the parcel
@@ -417,7 +521,9 @@ mod test {
         parcels.insert(sha_string.clone(), parcel_data);
 
         // Save
-        let writer = StandaloneWrite::new(&dir.path(), &id).expect("Create a writer");
+        let writer = StandaloneWrite::new(&dir.path(), &id)
+            .await
+            .expect("Create a writer");
         writer
             .write(inv, parcels)
             .await
@@ -482,4 +588,97 @@ mod test {
         // Otherwise, tmpfile will clean up the tmpdir too soon.
         dir.close().expect("deleted temp dir");
     }
+
+    #[tokio::test]
+    async fn should_round_trip_tarball() {
+        // Create temp dir
+        let dir = tempdir().expect("create a temp dir");
+        // Create invoice
+        let id: Id = "standalone/roundtrip/1.0.0"
+            .parse()
+            .expect("expect valid ID");
+        let mut inv = Invoice::new(BindleSpec {
+            id: id.clone(),
+            description: Some("testing standalone bindle".to_owned()),
+            authors: None,
+        });
+
+        // Create parcel
+        let parcel_data = "I'm a test fixture".as_bytes();
+        let sha = Sha256::digest(parcel_data);
+        let sha_string = format!("{:x}", sha);
+
+        // Add parcel
+        inv.parcel = Some(vec![Parcel {
+            label: Label {
+                name: "fixture.txt".to_owned(),
+                media_type: "text/plain".to_owned(),
+                size: parcel_data.len() as u64,
+                sha256: sha_string.clone(),
+                annotations: None,
+                origin: None,
+                feature: None,
+            },
+            conditions: None,
+        }]);
+        let mut parcels = HashMap::new();
+        parcels.insert(sha_string.clone(), parcel_data);
+
+        // Save
+        let writer = StandaloneWrite::new(&dir.path(), &id)
+            .await
+            .expect("Create a writer");
+        writer
+            .write(inv, parcels)
+            .await
+            .expect("Write parcel to disk");
+
+        // Tarball it
+        let output_dir = tempdir().expect("unable to create tempdir");
+        writer
+            .tarball(output_dir.path())
+            .await
+            .expect("Tarball should write to disk");
+
+        // Check that the tarball exists with the given name
+        let tarball_path = output_dir.path().join(format!("{}.tar.gz", id.sha()));
+        {
+            let md = tokio::fs::metadata(&tarball_path)
+                .await
+                .expect("stat tarball path");
+            assert!(md.is_file());
+        }
+
+        // Now attempt to read it
+        let reader = StandaloneRead::new_from_tarball(&tarball_path)
+            .await
+            .expect("construct a reader");
+
+        {
+            let md = tokio::fs::metadata(&reader.invoice_file)
+                .await
+                .expect("stat invoice path");
+            assert!(md.is_file());
+        }
+
+        // Verify that the parcel directory exists
+        {
+            let md = tokio::fs::metadata(&reader.parcel_dir)
+                .await
+                .expect("stat parcel path");
+            assert!(md.is_dir());
+        }
+
+        // Verify that the parcel exists
+        for p in reader.parcels.iter() {
+            tokio::fs::metadata(p)
+                .await
+                .unwrap_or_else(|e| panic!("failed to find {}: {}", p.display(), e));
+        }
+
+        // This keeps dir from being deleted until we are done with the test.
+        // Otherwise, tmpfile will clean up the tmpdir too soon.
+        dir.close().expect("deleted temp dir");
+        output_dir.close().expect("deleted temp output dir");
+    }
 }
